################################################## piRPred scriptR ###################################################
# V 1.0
# Brayet Jocelyn
# Ibisc / Genethon
# 04/2014
#
#
# Building position kernel
#

# Variables :
# The path is the unzipped folder
setwd("//home//pbatista//Desktop//piRPred_webserver")




kernelmatrix <- function(X,kernel,distance){

# kernelmatrix - construct a kernel matrix - Farida Zehraoui - IBISC
# 
#  K = kernelmatrix(X,kernel,arg)
# 
#  The inputs are
#    X       	- an n-by-Nx matrix of n real data points (of dimension Nx)
#    kernel  	- indicates which kernel function to use:
#                            * 1 : linear kernel
#                            * 2 : Gaussian kernel
#                            * 3 : polynomial kernel
#    distance  	- the distance measure to be used. This must be one of "euclidean", "maximum", "manhattan", "canberra", "binary" or "minkowski"
#    arg     	- parameter of the kernel function
#                            * linear kernel     :  k(x1,x2) = x1'*x2 (value of arg doesn't matter)
#                            * Gaussian kernel   :  k(x1,x2) = exp(-0.5*(x1 - x2)'*(x1 - x2)/arg)      --- if arg is a scalar
#                                                   k(x1,x2) = exp(-(x1 - x2)'*diag(arg)*(x1 - x2))    --- if arg is a vector
#                            * polynomial kernel :  k(x1,x2) = (1 + x1'*x2)^arg
# 
#  and the output is
#    K       	- the corresponding Gram matrix : Kij = k(X(i,:),X(j,:))
 	arg=0
 
	# number of data points
	n <-  nrow(X);

	# initialize K
	K <- matrix(0, nrow=n, ncol=n);
 
	# build K, depending on the kernel
	if(kernel==1){
		K <- X%*%t(X);

	}

	if(kernel==2){
		if(length(arg)==1){
    			mat <- as.matrix(dist(X, method = distance, diag = TRUE, upper = TRUE), nrow = n, ncol = n)
			mat[is.na(mat)]<-Inf
			med=median(mat^2)
			#print(med)
			arg=-log(0.5)/med
        		
			distance=0
			nombreSeq=n


			for(i in 1:50){

				#print("Argument : ")
				#print(arg)
	
				K <- exp(-(arg*(mat^2)))

				valPositif<-mean(K[1:trunc(n/2),1:trunc(n/2)])/2
				valNegatif<-mean(K[(trunc(n/2)+1):nombreSeq,(trunc(n/2)+1):nombreSeq])/2

				distanceInterCluster<-dist(c(valPositif,valNegatif),method = "euclidean")

				#print("Distance : ")
				#print(distanceInterCluster)

				arg=arg+arg/10

				if (distance<distanceInterCluster){
					distance=distanceInterCluster
				}else{
					break
				}

			}	
        	}else {
    			mat <- as.matrix(dist(X, method = distance, diag = TRUE, upper = TRUE), nrow = n, ncol = n)
        		K <- exp(-mat^2)
		
		}
        }
 
	if(kernel==3){
		K <- (1+X%*%t(X))^arg
	}
    
	return(K)
    
}


doNoyauSequence <-function(){

	data<-read.table("./Train/pos_train.txt",row.names=1)
	
	n <-  nrow(data)

	datatest<-read.table("./Test/pos_test_2_ordre.txt",row.names=1)

	data <- rbind(data,datatest)
	
	matrixkmer<-kernelmatrix(data,2,"euclidean")

	write.table(matrixkmer[(n+1):length(data[,1]),(1:n)],"./pos_test_2.txt", sep = " ", quote = FALSE,row.names = FALSE,col.names=FALSE)

}




#print(getwd())
print("Position kernel")

doNoyauSequence()

print("Position kernel do")




